/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.chart.tests.engine.util;

import junit.framework.TestCase;
import org.eclipse.birt.chart.util.LUDecomposition;
import org.eclipse.birt.chart.util.Matrix;

public class LUDecompositionTest extends TestCase
{

	double[][] A;
	Matrix matrix;
	LUDecomposition lu;

	/**
	 * Construct and initialize any objects that will be used in multiple tests.
	 * 
	 */
	protected void setUp( ) throws Exception
	{
		super.setUp( );
		A = new double[3][3];
		for ( int i = 0; i < 3; i++ )
		{
			for ( int j = 0; j < 3; j++ )
			{
				A[i][j] = ( i + 1 ) * ( j + 1 );
			}
		}
		matrix = new Matrix( A );
		lu = new LUDecomposition( matrix );
	}

	/**
	 * Collect and empty any objects that are used in multiple tests. 
	 * 
	 */
	protected void tearDown( ) throws Exception
	{
		super.tearDown( );
		A = null;
		matrix = null;
		lu = null;
	}


	public void testDet( )
	{
		double d = lu.det( );
		assertTrue( d == -0.0 );
	}
	
	public void testIsNonsingular( )
	{
		assertFalse( lu.isNonsingular( ) );
	}

	public void testGetL( )
	{
		Matrix X = lu.getL( );

		for ( int i = 0; i < 3; i++ )
		{
			for ( int j = 0; j < 3; j++ )
			{
				if ( i == j )
				{
					assertTrue( X.get( i, j ) == 1.0 );
				}
				else if ( i < j )
				{
					assertTrue( X.get( i, j ) == 0.0 );
				}
				else
				{
					assertTrue( X.get( i, j ) == (double) ( 3 - i - j ) / 3 );
				}
			}
		}
	}

	public void testGetU( )
	{
		Matrix X = lu.getU( );

		for ( int i = 0; i < 3; i++ )
		{
			for ( int j = 0; j < 3; j++ )
			{
				if ( i > j )
				{
					assertTrue( X.get( i, j ) == 0.0 );
				}
				else if ( i == 0 )
				{
					assertTrue( X.get( i, j ) == 3 * ( j + 1 ) );
				}
				else
				{
					assertTrue( X.get( i, j ) == 0.0 );
				}
			}
		}
	}

	public void testGetPivot( )
	{
		int[] a = lu.getPivot( );
		for ( int i = 0; i < 3; i++ )
		{
			assertTrue( a[i] == ( 2 - i ) );
		}
	}

	public void testGetDoublePivot( )
	{
		double[] a = lu.getDoublePivot( );
		for ( int i = 0; i < 3; i++ )
		{
			assertTrue( a[i] == (double) ( 2 - i ) );
		}
	}
}