/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.chart.tests.engine.util;

import junit.framework.TestCase;
import org.eclipse.birt.chart.util.Matrix;

public class MatrixTest extends TestCase {
	
	double[][] A;
	int[] a;
	Matrix matrix, subMatrix;
	
	/**
	 * Construct and initialize any objects that will be used in multiple tests.
	 * 
	 */
	protected void setUp() throws Exception {
		super.setUp();
		A = new double[3][3];
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				A[i][j] = (i+1) * (j+1);
			}
		}		
		matrix = new Matrix(A);
		subMatrix = new Matrix( 1, 1, 1.0);
		a = new int[1];
		a[0] = 0;
	}
	
	/**
	 * Collect and empty any objects that are used in multiple tests. 
	 * 
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
		A = null;
		matrix = null;
	}
	
	public void testHypot() {
		assertTrue(Matrix.hypot( -4, 3) == 5);
		assertTrue(Matrix.hypot( 3, 4) == 5);
		assertTrue(Matrix.hypot( 0, 0) == 0);
	}
	
	public void testConstructWithCopy() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.get(i,j) == A[i][j]);
			}
		}
	}
	
	public void testCopy(){
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.copy().get(i,j) == matrix.get(i,j));
			}
		}
	}
	
	public void testGetArray() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.getArray()[i][j] == A[i][j]);
			}
		}
	}
	
	public void testGetArrayCopy() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.getArrayCopy()[i][j] == A[i][j]);
			}
		}
	}
	
	public void testGetColumnPackedCopy() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.getColumnPackedCopy()[i*3+j] == A[j][i]);
			}
		}
	}
	
	public void testGetRowPackedCopy() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.getRowPackedCopy()[i+j*3] == A[j][i]);
			}
		}
	}

	public void testGetRowDimension() {
		assertEquals(3, matrix.getRowDimension());
	}
	
	public void testGetColumnDimension() {
		assertEquals(3, matrix.getColumnDimension());
	}
	
	public void testGetMatrix(){
		assertTrue(matrix.getMatrix(0,0,0,0).get(0,0) == subMatrix.get(0,0));
		assertTrue(matrix.getMatrix(a, a).get(0,0) == subMatrix.get(0,0));
		assertTrue(matrix.getMatrix(0, 0, a).get(0,0) == subMatrix.get(0,0));
		assertTrue(matrix.getMatrix(a, 0, 0).get(0,0) == subMatrix.get(0,0));
	}
	
	public void testTranspose() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.transpose().get(i, j) == A[j][i]);
			}
		}
	}
	
	public void testNorm1() {
		assertTrue( matrix.norm1() == 18.0 );
	}
	
	public void testNormF() {
		assertTrue( matrix.normF() == 14.0 );
	}
	
	public void testUminus() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.uminus().get(i, j) == -A[i][j]);
			}
		}
	}
	
	public void testPlus() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.plus(matrix).get(i, j) == A[i][j] * 2);
			}
		}
	}
	
	public void testMinus() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.minus(matrix).get(i, j) == 0.0);
			}
		}
	}
	
	public void testArrayTimes() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.arrayTimes(matrix).get(i, j) == A[i][j] * A[i][j]);
			}
		}
	}
	
	public void testRightDivide() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.arrayRightDivide(matrix).get(i, j) == 1.0 );
			}
		}
	}
	
	public void testLeftDivide() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.arrayLeftDivide(matrix).get(i, j) == 1.0 );
			}
		}
	}
	
	public void testTimes() {
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.times(2.0).get(i, j) == 2 * A[i][j] );
			}
		}
		
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				assertTrue(matrix.times(matrix).get(i, j) == 14 * A[i][j]);
			}
		}
	}
	
	public void testTrace() {
		assertTrue( matrix.trace() == 14.0 );
	}
	
	public void testIdentity() {
		Matrix A = Matrix.identity(3, 3);
		for (int i = 0; i< 3; i++){
			for (int j = 0; j < 3; j++){
				if (i == j){
				assertTrue(A.get(i, j) == 1.0 );
				} else {
				assertTrue(A.get(i, j) == 0.0 );
				}
			}
		}
	}
}
