/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.tests.designer.testutil;


import java.io.FileInputStream;


import junit.framework.TestCase;

import org.eclipse.birt.report.designer.core.model.SessionHandleAdapter;


import org.eclipse.birt.report.model.api.ReportDesignHandle;
import org.eclipse.birt.report.model.elements.ReportDesign;
import org.eclipse.core.runtime.Platform;

/**
 * Base class of unit tests
 * 
 *  
 */

public abstract class BaseTestCase extends TestCase
{

	
	protected static final String PLUGIN_NAME = "org.eclipse.birt.report.tests.designer";
	/*
	 *  The plugin location   
	 */
	protected static final String PLUGINLOC = Platform.getBundle(PLUGIN_NAME).getLocation();
	protected static final String PLUGIN_PATH =System.getProperty("user.dir")+ "/plugins/" + 
	            BaseTestCase.PLUGINLOC.substring(BaseTestCase.PLUGINLOC.indexOf("/")+1) + "bin/";
	protected static final String DESIGN_FILE_PATH = (PLUGIN_PATH + 
			PLUGIN_NAME.replace('.','/') + "/input/" + 
			ITestConstants.TEST_DESIGN_FILE).replace('/', 
					System.getProperty("file.separator").charAt(0));
	
	protected ReportDesignHandle report;

	/**
	 * Default constructor
	 */
	public BaseTestCase( )
	{//Do nothing
	}

	/**
	 * Constructor
	 * 
	 * @param name
	 */
	public BaseTestCase( String name )
	{
		super( name );
	}

	protected void setUp( ) throws Exception
	{
		loadFile( DESIGN_FILE_PATH );
		report = SessionHandleAdapter.getInstance( ).getReportDesignHandle( );
	}

	protected void tearDown( ) throws Exception
	{
		report = null;
	}

	private void loadFile( String fileName ) throws Exception
	{
		SessionHandleAdapter.getInstance( ).init( fileName,
				new FileInputStream(fileName));
	}


	/**
	 * Gets the report design for tests
	 * 
	 * @return the report design for tests
	 */
	protected ReportDesign getReportDesign( )
	{
		return report.getDesign( );
	}

	protected ReportDesignHandle getReportDesignHandle( )
	{
		return report;
	}

}