/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.tests.designer.ui;

import org.eclipse.birt.report.designer.ui.editors.ReportEditor;
import org.eclipse.birt.report.tests.designer.testutil.BirtUITestCase;
import org.eclipse.birt.report.tests.designer.testutil.FileUtil;
import org.eclipse.birt.report.tests.designer.testutil.PlatformUtil;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.ui.IEditorPart;

/**
 * Simple test for BirtUITestCase
 */

public class SimpleUITest extends BirtUITestCase
{

	/**
	 * 
	 * Test showPerpersite method
	 * 
	 * invoke showPerpersite then check the active perspective' ID and name to
	 * test if it works correctly
	 */

	public void testShowPerspective( ) throws Throwable
	{
		showPerspective( );
		assertEquals( PERSPECTIVE_ID, tPage.getPerspective( ).getId( ) );
		assertEquals( PERSPECTIVE_NAME, tPage.getPerspective( ).getLabel( ) );
	}


/**
 * @param int id the root id
 * @throws Exception
 */
	public void testOpenEditor( ) throws Exception
	{
		if ( PlatformUtil.isWindows( ) )
		{//platform related issue
			IProject p = FileUtil.createProject( TEST_PROJECT_NAME );
			IFile f = FileUtil.createFile( TEST_DESIGN_FILE, p );
			IEditorPart tPart = openEditor(p, f);
			assertTrue( tPart instanceof ReportEditor );
			assertEquals( tPart, tPage.getActiveEditor( ) );
			assertEquals( EDITOR_ID, tPart.getSite( ).getId( ) );
			assertEquals( EDITOR_NAME, tPart.getSite( ).getRegisteredName( ) );
			assertEquals( TEST_DESIGN_FILE, tPart.getTitle( ) );
			closeEditor( );
		}
	}

	public void testGetView( )
	{
		assertEquals( DATA_EXPLORER_NAME,
				getView( DATA_EXPLORER_ID ).getTitle( ) );
	}

}