/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.tests.designer.ui.extensions;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.eclipse.birt.report.designer.internal.ui.editors.schematic.LayoutEditor;
import org.eclipse.birt.report.designer.internal.ui.editors.schematic.editparts.LabelEditPart;
import org.eclipse.birt.report.designer.internal.ui.editors.schematic.figures.LabelFigure;
import org.eclipse.birt.report.designer.internal.ui.extension.ExtendedElementUIPoint;
import org.eclipse.birt.report.designer.internal.ui.extension.ExtendedUIAdapter;
import org.eclipse.birt.report.designer.internal.ui.extension.ExtensionPointManager;
import org.eclipse.birt.report.designer.ui.editors.ReportEditor;
import org.eclipse.birt.report.designer.ui.extensions.IReportItemBuilderUI;
import org.eclipse.birt.report.designer.ui.extensions.IReportItemFigureProvider;
import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.ExtendedItemHandle;
import org.eclipse.birt.report.model.api.extension.IReportItem;
import org.eclipse.birt.report.tests.designer.testutil.BirtUITestCase;
import org.eclipse.birt.report.tests.designer.testutil.FileUtil;
import org.eclipse.birt.report.tests.designer.testutil.ITestConstants;
import org.eclipse.birt.report.tests.designer.ui.extension.impl.ExtendedElement;
import org.eclipse.birt.report.tests.designer.ui.extension.impl.ReportItemPropertyEditUIImpl;
import org.eclipse.birt.report.tests.designer.ui.extension.impl.ReportItemFigureUIImpl;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IEditorPart;


/**
 *  
 */

public class IReportItemUITest extends BirtUITestCase
{

	private IReportItemFigureProvider ui;
	
	public static Test suite(){
		return new TestSuite(IReportItemUITest.class);
		
	}

	protected void setUp( ) throws Exception
	{
		super.setUp( );
		ui = ExtensionPointManager.getInstance( )
				.getExtendedElementPoint( ITestConstants.TEST_EXTENSION_NAME )
				.getReportItemUI( );
	}

	public void testGetQuickEditPage( )
	{
		assertTrue( ExtensionPointManager.getInstance( ).getExtendedElementPoint( ITestConstants.TEST_EXTENSION_NAME ).getReportItemPropertyEditUI( ) instanceof ReportItemPropertyEditUIImpl);
	}

	public void testGetBuilder( )
	{
		ExtendedElementUIPoint point = ExtensionPointManager.getInstance( ).getExtendedElementPoint( ITestConstants.TEST_EXTENSION_NAME );
			
		IReportItemBuilderUI builder = point.getReportItemBuilderUI( ) ;

		
		assertNotNull( builder );
	
		ExtendedItemHandle handle = (ExtendedItemHandle)getReportDesignHandle( ).findElement("test");
		assertNotNull( handle );
		assertEquals( ITestConstants.TEST_EXTENSION_NAME,
				handle.getExtensionName( ) );

//		
		try{
			handle.loadExtendedElement();
			
		}catch(Exception ex){
			ex.printStackTrace();
			
		}
	//	assertEquals( Window.OK, builder.open( handle ) );
		for ( int i = 0; i < ReportItemFigureUIImpl.TEST_ELEMENT_CONTENT.length; i++ )
		{
			assertEquals( ReportItemFigureUIImpl.TEST_ELEMENT_CONTENT_LOADED[i],
					handle.getProperty( ReportItemFigureUIImpl.TEST_PROPERTY[i] ) );
		}
		
//		assertEquals( Window.CANCEL, builder.open( null ) );
		assertEquals( Window.OK, builder.open( handle ) );
//		for ( int i = 0; i < ReportItemFigureUIImpl.TEST_ELEMENT_CONTENT.length; i++ )
//		{
//			assertEquals( ReportItemFigureUIImpl.TEST_ELEMENT_CONTENT[i],
//					handle.getProperty( ReportItemFigureUIImpl.TEST_PROPERTY[i] ) );
//		}
//		assertEquals( Window.OK, builder.open( handle ) );
		for ( int i = 0; i < ReportItemFigureUIImpl.TEST_ELEMENT_CONTENT.length; i++ )
		{
			assertEquals( ReportItemFigureUIImpl.TEST_ELEMENT_CONTENT_EDITED[i],
					handle.getProperty( ReportItemFigureUIImpl.TEST_PROPERTY[i] ) );
		}
		
		
	}

	/**
	 * 
	 * 
	 * @throws Exception
	 * @test purpose  this method is to test the extended UI figure 
	 *     1. whether the figure can be correctly obtained.
	 *     2. whether the figure is rendered to the editor.
	 * 
	 * @assertion: 
	 *     1. the figure created should be an instance of LabelFigure.
	 *     2. the element handle obtained from the report handle should
	 *        be an instance of extendedItemHandle;
	 *     3. The Label text obtained should be identical to what has been set into
	 * 
	 * 
	 *
	 */
	
	
	public void testGetFigure( ) throws Exception 
	{
		ExtendedElementUIPoint point = ExtensionPointManager.getInstance( ).getExtendedElementPoint( ITestConstants.TEST_EXTENSION_NAME );
		IReportItemFigureProvider provider = point.getReportItemUI();
	
		
		assertTrue(provider instanceof ExtendedUIAdapter);
		ExtendedUIAdapter exdProvider = (ExtendedUIAdapter)provider;
		
		
		DesignElementHandle handle = report.findElement("test");	
		assertTrue("Type mismatch", handle instanceof ExtendedItemHandle);
		
	
		ExtendedItemHandle exdHandle = (ExtendedItemHandle)handle;
		IFigure figure = exdProvider.createFigure(exdHandle);
			
		IReportItem item = null;
		
		try{
			
			item = exdHandle.getReportItem();
		}catch(Exception ex){
			fail("Extensions cannot be loaded");
		}
		assertNotNull("should not be null", item);
		assertTrue("Type Mismatch", item instanceof ExtendedElement);
		
		
		ExtendedElement exdItem = (ExtendedElement)item;
		

			
			showPerspective();
			IProject p = FileUtil.createProject( TEST_PROJECT_NAME );
			IFile f = FileUtil.createFile(TEST_DESIGN_FILE,p);
			IEditorPart part = openEditor(p,f);
			
			assertTrue( part instanceof ReportEditor );
			
			ReportEditor rEditor = (ReportEditor)part;
//		
//		
//		
//			
//			LayoutEditor editor = new LayoutEditor(rEditor);
//		
//			EditPart pt = (EditPart)editor.getReportDesigner();
//		    
//			assertTrue("Type Mismatch", pt instanceof LabelEditPart);
//			LabelEditPart lp = (LabelEditPart)pt;
			
			assertTrue("Type Mismatch", figure instanceof LabelFigure);
			LabelFigure fg = (LabelFigure)figure;
			
			String str = handle.getProperty( ReportItemFigureUIImpl.TEST_PROPERTY[0] )
			             + ":"
			             + exdHandle.getProperty( ReportItemFigureUIImpl.TEST_PROPERTY[1] ).toString( );
			
			str = str.trim();

			assertEquals("Label Texts should be equal", str, fg.getText());
			
			
			
	    
		
		
		
		
	    	
		
		
	}
	


	public void testUpdateFigure( )
	{		
		try
		{
			ui.updateFigure( null, null );
		}
		catch ( Throwable e )
		{//should invoke exception
			return;
		}
		fail( );
	}
}