/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.tests.designer.ui.odadatasource.wizard;

import java.nio.charset.Charset;
import java.util.SortedMap;

import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage;


public class TestDataSourcePropertyPage extends AbstractPropertyPage{
	
	private transient Table resultSetTable = null;
	private transient Text folderLocation = null;
	private transient Button typeLineCheckBox = null;
	private transient Combo charSetSelectionCombo = null;
	private static String DEFAULT_MESSAGE = Messages.getString("wizard.defaultMessage.selectFolder"); //$NON-NLS-1$
	private SortedMap charSetMap = null;
    public TestDataSourcePropertyPage(String pageName)
    {
 
    }

	public boolean canLeave()
	{
		return false;
	}
	
    public Control createPageControl(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 3;
        composite.setLayout(layout);
        
        GridData data;
		setupFolderLocation(composite);
                
        setupCharset(composite);
        
        setupTypeLineCheckBox(composite);
 
        
        return composite;
    }

	private void setupTypeLineCheckBox(Composite composite) {
		
		typeLineCheckBox = new Button(composite, SWT.CHECK);
        GridData data = new GridData();
        data.horizontalSpan = 3;
        typeLineCheckBox.setLayoutData(data);
        typeLineCheckBox.setText(Messages.getString("label.includeTypeLine"));
	}

	public void pageActivated( )
	{
		getContainer( ).setMessage( Messages.getString( "dataset.editor.preview" ),//$NON-NLS-1$
				IMessageProvider.NONE );

	}
    public String getFolderLocation()
    {
        return folderLocation.getText();
    }
    public String getWhetherUseSecondLineAsTypeLine()
    {
    	return typeLineCheckBox.getSelection()?"YES":"NO";
    }
   
    public String getCharSet()
    {
    	String key = charSetSelectionCombo.getItem(charSetSelectionCombo.getSelectionIndex());
    	return key;
    }
	public void setupFolderLocation(Composite composite) {
		
		Label label = new Label(composite, SWT.NONE);
        label.setText(Messages.getString("label.selectFolder")); //$NON-NLS-1$
        
        GridData data = new GridData(GridData.FILL_HORIZONTAL);

        folderLocation = new Text(composite, SWT.BORDER);
        folderLocation.setLayoutData(data);
        folderLocation.setText("New test item");
 	}
	private void setupCharset(Composite composite) {
		
		Label labelCharSet = new Label(composite, SWT.NONE);
        labelCharSet.setText(Messages.getString("label.selectCharset")); //$NON-NLS-1$
        
        charSetSelectionCombo = new Combo(composite, SWT.READ_ONLY);
            
        GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
        data.horizontalSpan = 2;
        charSetSelectionCombo.setLayoutData( data );
        
        charSetMap = Charset.availableCharsets();
        Object[] charSetsArray = charSetMap.keySet().toArray();
        for(int i = 0; i < charSetsArray.length; i++)
        {
        	String charSetName = ((Charset) charSetMap.get( charSetsArray[i] )).name();
        	charSetSelectionCombo.add( charSetName );
        	if( Constants.ODA_DEFAULT_CHARSET.equalsIgnoreCase( charSetName ))
        		charSetSelectionCombo.select(i);
        }
	}
}
