/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.tests.engine;

import java.util.Vector;
import java.util.logging.*;
import java.io.*;
import java.beans.*;

import junit.framework.TestCase;

/**
 * Put your comments @
 * @version $Revision: 1.2 $Date: 2005-6-28
 */
public class CallSequence {

    private static int level = 1;

    private boolean record;

    private String fileName;

    Vector callList = new Vector();

    int currentPoint;

    public CallSequence(boolean record, String filePath) {
        this.record = record;
        this.fileName = filePath;
        if (!record) {
            //init the call list to verify the current invoke sequence.
            initCallList();
        }

        String lev = System.getProperty("CALL_RECORD_LEVEL");
        if (lev != null) {
            System.out.println("Call level set to : " + lev);
            level = Integer.parseInt(lev);
        }
    }

    private void initCallList() {
        XMLDecoder d;
        try {
            d = new XMLDecoder(new BufferedInputStream(new FileInputStream(
                    fileName)));
            callList = (Vector) d.readObject();
            d.close();
        } catch (FileNotFoundException e) {
            //            throw new RuntimeException(e);

        }
    }

    public void called(Object[] params) {
        if (record) {
            recordCalled(params);
        } else {
            verifyCalled(params);
        }
    }

    private void verifyCalled(Object[] params) {

        if (callList == null) {
            return;
        }
        Throwable t = new Throwable();
        StackTraceElement[] ts = t.getStackTrace();
        //		System.out.println("method : " + ts[3].getMethodName()
        //				+ " | with paramenter : ");
        //		for (int i = 0; i < param.length; i++) {
        //			//System.out.println(getGetValue(param[i]));
        //		}

        TestCase.assertTrue(currentPoint < callList.size());

        Object[] oneCall = (Object[]) callList.elementAt(currentPoint++);
        if (oneCall.length == 4) {
            //            TestCase.assertEquals(oneCall[0], ts[3].getClassName());
            //            TestCase.assertEquals(oneCall[2], ts[3].getFileName());
            TestCase.assertEquals(oneCall[3], ts[3].getMethodName());
            MethodInvoke mi = new MethodInvoke(null, ts[3].getMethodName(),
                    params, level);
            TestCase.assertTrue(mi.match((Vector) oneCall[3]));
            //            System.out.println(mi.match((Vector) oneCall[3]));
        }
    }

    private void recordCalled(Object[] params) {
        Throwable t = new Throwable();
        StackTraceElement[] ts = t.getStackTrace();
        //		System.out.println("method : " + ts[3].getMethodName()
        //				+ " | with paramenter : ");
        //		for (int i = 0; i < param.length; i++) {
        //			//System.out.println(getGetValue(param[i]));
        //		}
        Object[] oneCall = new Object[4];
        oneCall[0] = ts[3].getClassName();
        oneCall[1] = ts[3].getFileName();
        oneCall[2] = ts[3].getMethodName();
        MethodInvoke mi = new MethodInvoke(null, ts[3].getMethodName(), params,
                level);
        oneCall[3] = mi.getInvokeAsStringVec();
        callList.add(oneCall);
    }

    public void finished() {
        //write it to xml file.
        if (record) {
            XMLEncoder e;

            try {
                e = new XMLEncoder(new BufferedOutputStream(
                        new FileOutputStream(fileName)));

                e.writeObject(callList);
                e.close();
            } catch (FileNotFoundException e1) {
                throw new RuntimeException(e1);
            }
        } else {
            //nothing
        }
    }

}