/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.tests.engine;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Vector;

/**
 * Put your comments @
 * @version $Revision: 1.2 $Date: 2005-6-28
 */
public class MethodInvoke {

    private final String ROOT_CALL = "call from sky";

    private int level = 1;

    private String methodName;

    private Object params[];

    private Object target;

    private HashSet visited = new HashSet();

    public MethodInvoke(Object target, String methodName, Object[] params) {
        this.target = target;
        this.methodName = methodName;
        this.params = params;
    }

    public MethodInvoke(Object target, String methodName, Object[] params,
            int level) {
        this(target, methodName, params);
        this.level = level;
    }

    public boolean match(Vector paras) {
        boolean result = true;
        for (int i = 0; i < paras.size(); i++) {
            MethodValue tmv = (MethodValue) paras.elementAt(i);
            if (!match(tmv, params[i])) {
                result = false;

                //continue the compare,
            }
        }
        return result;
    }

    protected boolean match(MethodValue mv, Object o) {
        boolean result = true;
        System.out.println(mv.getValue());
        if (mv.isFinished() && !mv.isPrimary()) {
            //ignored about this check.
        } else if (mv.isPrimary() && Utils.isPrimary(o)) {
            if ((o == null && mv.getValue() == null)
                    || (o != null && mv.getValue() != null && o.equals(mv
                            .getValue()))) {

            } else {
                result = false;
            }
        } else {
            if (mv.isPrimary() || Utils.isPrimary(o)) {
                result = false;
            } else {
                if (!mv.isFinished()) {
                    Vector tv = (Vector) mv.getValue();
                    for (int i = 0; i < tv.size(); i++) {
                        MethodValue tmv1 = (MethodValue) tv.elementAt(i);
                        try {
                            Method m = o.getClass().getMethod(
                                    tmv1.getMethodName(), new Class[] {});
                            Object to = m.invoke(o, new Object[] {});
                            result = match(tmv1, to);
                            if (!result) {
                                break;
                            }
                        } catch (Exception e) {
                            result = false;
                            e.printStackTrace();
                        }
                    }
                } else {
                    //is finished and not primary. Reco it as matched.
                    result = true;
                }
            }
        }
        return result;
    }

    public Vector getInvokeAsStringVec() {
        Vector result = new Vector();
        for (int i = 0; i < params.length; i++) {
            result.add(getMV(params[i], ROOT_CALL, level - 1));
        }
        return result;
    }

    /**
     * Get the method vlue pares tree.
     * 
     * @param obj
     * @param methodName
     * @param level
     * @return
     */
    protected MethodValue getMV(Object obj, String methodName, int level) {
        MethodValue result = new MethodValue();

        result.setMethodName(methodName);
        if (obj == null) {
            result.setFinished(true);
            result.setPrimary(true);
        } else {
            result.setClassName(obj.getClass().getName());
        }

        if (Utils.isPrimary(obj)) {
            result.setPrimary(true);
            result.setValue(obj);
            result.setFinished(true);
        } else {
            result.setPrimary(false);
            if (level < 0 || visited.contains(obj)) {
                //finished
                result.setFinished(true);
                result.setValue(obj.toString());
            } else {
                visited.add(obj);

                if (!Utils.isPrimary(obj)) {

                    //get all the get methods and related values.,
                    Vector mvs = new Vector();
                    result.setValue(mvs);

                    Method ms[] = obj.getClass().getMethods();
                    for (int i = 0; i < ms.length; i++) {
                        if (ms[i].getName().startsWith("get")
                                && (ms[i].getParameterTypes().length == 0)) {
                            try {
                                Object value = ms[i].invoke(obj,
                                        new Object[] {});
                                mvs
                                        .add(getMV(value, ms[i].getName(),
                                                level - 1));
                            } catch (Exception e) {
                                MethodValue tmv = new MethodValue();
                                tmv.setFinished(true);

                                tmv.setValue("Error in get method invoke");
                                tmv.setMethodName(ms[i].getName());
                                tmv.setPrimary(false);
                                mvs.add(tmv);
                            }
                        }
                    }
                }

            }
        }

        return result;
    }
}