/*******************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.tests.data.engine.smoke;

import org.eclipse.birt.tests.data.DataTestCase;
import org.eclipse.birt.data.engine.api.IBaseExpression;
import org.eclipse.birt.data.engine.api.IPreparedQuery;
import org.eclipse.birt.data.engine.api.IQueryResults;
import org.eclipse.birt.data.engine.api.IResultIterator;
import org.eclipse.birt.data.engine.api.querydefn.BaseTransform;
//import org.eclipse.birt.data.engine.api.querydefn.FilterDefinition;
//import org.eclipse.birt.data.engine.api.querydefn.GroupDefinition;
import org.eclipse.birt.data.engine.api.querydefn.OdaDataSetDesign;
import org.eclipse.birt.data.engine.api.querydefn.QueryDefinition;
import org.eclipse.birt.data.engine.api.querydefn.ScriptExpression;
//import org.eclipse.birt.data.engine.api.querydefn.SortDefinition;
import testutil.ConfigText;

/**
 * DtE features test.
 */
public class DsAggregationTest extends DataTestCase
{

	/*
	 * @see org.eclipse.birt.data.engine.api.APITestCase#getDataSourceInfo()
	 */
	protected DataSourceInfo getDataSourceInfo( )
	{
		return new DataSourceInfo( ConfigText.getString( "Api.TestData.TableName" ),
				ConfigText.getString( "Api.TestData.TableSQL" ),
				ConfigText.getString( "Api.TestData.TestDataFileName" ) );
		
	}

 
	 	public void test_RunningAggregationExpression() throws Exception
	{
		// Test a SQL with duplicate column name (quite common with join data sets)
				System.out.println("mj");
		String testSQL =  "select COUNTRY, AMOUNT from " + getTestTableName( );
		((OdaDataSetDesign)this.dataSet).setQueryText( testSQL );
		System.out.println(testSQL);
		
		IBaseExpression[] expressions = new IBaseExpression[]{
				new ScriptExpression( "row.COUNTRY", 0 ),
				new ScriptExpression( "Total.runningSum(row.AMOUNT)", 2 ) };

		// define a query design				
		QueryDefinition queryDefn = newReportQuery( );
		queryDefn.addExpression(expressions[0], BaseTransform.ON_EACH_ROW);
		queryDefn.addExpression(expressions[1], BaseTransform.AFTER_LAST_ROW);	
		
		IPreparedQuery preparedQuery = dataEngine.prepare( queryDefn );
		IQueryResults queryResults = preparedQuery.execute( null );
		IResultIterator resultIt = queryResults.getResultIterator( );
		assertTrue( resultIt.next() );

		resultIt.getValue( expressions[0] );
		resultIt.getValue( expressions[1] );
		outputQueryResult( executeQuery( queryDefn ), expressions );
		checkOutputFile();
		
	}
	
	
	/**
	 * Test feature of
	 * 		aggregation expression 
	 */
	public void test_AggregationExpression() throws Exception
	{
		// Test a SQL with duplicate column name (quite common with join data sets)
		String testSQL =  "select COUNTRY, AMOUNT from " + getTestTableName( );
		((OdaDataSetDesign)this.dataSet).setQueryText( testSQL );
		
		IBaseExpression[] expressions = new IBaseExpression[]{
				new ScriptExpression( "row.COUNTRY", 0 ),
				new ScriptExpression( "Total.sum(row.AMOUNT)", 2 ) };
		
		// define a query design				
		QueryDefinition queryDefn = newReportQuery( );
		queryDefn.addExpression(expressions[0], BaseTransform.ON_EACH_ROW);
		queryDefn.addExpression(expressions[1], BaseTransform.AFTER_LAST_ROW);	
		
		IPreparedQuery preparedQuery = dataEngine.prepare( queryDefn );
		IQueryResults queryResults = preparedQuery.execute( null );
		IResultIterator resultIt = queryResults.getResultIterator( );
		assertTrue( resultIt.next() );

		resultIt.getValue( expressions[0] );
		resultIt.getValue( expressions[1] );
		System.out.print( resultIt.getValue( expressions[1] ) );
	}

	/**
	 * Create query definition from passed parameteres
	 * and then run it.
	 * @param expressions
	 * @param groupDefn
	 * @param sortDefn
	 * @param filters
	 * @throws Exception
	 */
	/*private void createAndRunQuery( IBaseExpression[] expressions,
			GroupDefinition[] groupDefn, SortDefinition[] sortDefn, 
			FilterDefinition[] filters ) throws Exception
	{
		QueryDefinition queryDefn = createQueryDefn( expressions,
				groupDefn, sortDefn, filters );
		outputQueryResult( executeQuery( queryDefn ), expressions );
	}

	/**
	 * Create query definition from passed parameters
	 * @param expressions row expression
	 * @param groupDefn 
	 * @param sortDefn
	 * @param filters
	 * @return QueryDefinition
	 */
	/*private QueryDefinition createQueryDefn( IBaseExpression[] expressions,
			GroupDefinition[] groupDefn, SortDefinition[] sortDefn, FilterDefinition[] filters )
	{
		// define a query design		
		QueryDefinition queryDefn = newReportQuery( );
		
		if ( groupDefn != null )
			for ( int i = 0; i < groupDefn.length; i++ )
				queryDefn.addGroup( groupDefn[i] );
		if ( sortDefn != null )
			for ( int i = 0; i < sortDefn.length; i++ )
				queryDefn.addSort( sortDefn[i] );
		if ( expressions != null )
			for ( int i = 0; i < expressions.length; i++ )
				queryDefn.addExpression( expressions[i],
						BaseTransform.ON_EACH_ROW );
		if ( filters != null )
			for ( int i = 0; i < filters.length; i++ )
				queryDefn.addFilter( filters[i] );
		
		return queryDefn;
	}*/

	
}